@@ -2,8 +2,8 @@ from ClassHierarchySimple import ClassHierarchySimple
 from ClassHierarchyGraph import ClassHierarchyGraph
 from InheritanceFormatter import InheritanceFormatter
 from HeadingFormatter import HeadingFormatter
-from DetailFormatter import DetailFormatter
-from SummaryFormatter import SummaryFormatter
+from DeclarationFormatter import DeclarationDetailFormatter
+from DeclarationFormatter import DeclarationSummaryFormatter
 from SummaryCommenter import SummaryCommenter
 from DetailCommenter import DetailCommenter
 from SourceLinker import SourceLinker
