@@ -135,13 +135,14 @@ SWalker::add_comments(AST::Declaration* decl, PTree::Node *node)
 {
   if (!node) return;
   
-  AST::Comment::vector comments;
+  std::vector<std::string> comments;
 
   // First, make sure that node is a list of comments
   if (PTree::type_of(node) == Token::ntDeclaration)
     node = static_cast<PTree::Declaration*>(node)->get_comments();
 
   // Loop over all comments in the list
+  bool suspect = false;
   for (PTree::Node *next = PTree::rest(node); node && !node->is_atom(); next = PTree::rest(node))
   {
     PTree::Node *first = PTree::first(node);
@@ -150,8 +151,6 @@ SWalker::add_comments(AST::Declaration* decl, PTree::Node *node)
       node = next;
       continue;
     }
-    update_line_number(node);
-
     // Check if comment is continued, eg: consecutive C++ comments
     while (next && PTree::first(next) && PTree::first(next)->is_atom())
     {
@@ -183,7 +182,7 @@ SWalker::add_comments(AST::Declaration* decl, PTree::Node *node)
     // all comments that are not immediately (i.e. separated
     // by a single new line) followed by a declaration are
     // marked as 'suspect'
-    bool suspect = false;
+    suspect = false;
     const char *pos = first->position() + first->length();
     while (*pos && strchr(" \t\r", *pos)) ++pos;
     if (*pos == '\n')
@@ -195,23 +194,17 @@ SWalker::add_comments(AST::Declaration* decl, PTree::Node *node)
     }
 
     if (decl)
-    {
-      AST::Comment* comment = make_Comment(my_file, my_lineno, first, suspect);
-      comments.push_back(comment);
-    }
-    if (my_links) my_links->long_span(first, "file-comment");
+      comments.push_back(PTree::reify(first));
+    if (my_links) my_links->long_span(first, "comment");
     // Set first to 0 so we dont accidentally do them twice (eg:
     // when parsing expressions)
     node->set_car(0);
     node = next;
   }
-
+  if (suspect) comments.push_back("");
   // Now add the comments, if applicable
   if (decl)
-    for (AST::Comment::vector::iterator i = comments.begin();
-         i != comments.end();
-         ++i)
-      decl->comments().push_back(*i);
+    decl->comments() = comments;
 }
 
 // -- These methods implement add_comments for various node types that store
@@ -236,7 +229,7 @@ void SWalker::find_comments(PTree::Node *node)
 {
   PTree::Node *leaf, *parent;
   leaf = FindLeftLeaf(node, parent);
-  if (leaf) add_comments(0, dynamic_cast<PTree::CommentedAtom *>(leaf));
+  if (leaf) add_comments(0, static_cast<PTree::CommentedAtom *>(leaf));
 }
 
 PTree::Node *SWalker::translate_arg_decl_list(bool, PTree::Node *, PTree::Node *)
@@ -350,7 +343,7 @@ void SWalker::visit(PTree::Atom *node)
   if (*str >= '0' && *str <= '9' || *str == '.')
   {
     // Assume whole node is a number
-    if (my_links) my_links->span(node, "file-number");
+    if (my_links) my_links->span(node, "literal");
     // TODO: decide if Long, Float, Double, etc
     const char* num_type = (*str == '.' ? "double" : "int");
     while (*++str)
@@ -392,13 +385,13 @@ void SWalker::visit(PTree::Atom *node)
   else if (*str == '\'')
   {
     // Whole node is a char literal
-    if (my_links) my_links->span(node, "file-string");
+    if (my_links) my_links->span(node, "string");
     my_type = my_lookup->lookupType("char");
   }
   else if (*str == '"')
   {
     // Assume whole node is a string
-    if (my_links) my_links->span(node, "file-string");
+    if (my_links) my_links->span(node, "string");
     my_type = my_lookup->lookupType("char");
     Types::Type::Mods pre, post;
     pre.push_back("const");
@@ -411,7 +404,7 @@ void SWalker::visit(PTree::Atom *node)
     AST::Declaration* decl;
     update_line_number(node);
     decl = my_builder->add_tail_comment(my_lineno);
-    add_comments(decl, dynamic_cast<PTree::CommentedAtom *>(node));
+    add_comments(decl, static_cast<PTree::CommentedAtom *>(node));
   }
   else
   {
@@ -460,7 +453,7 @@ void SWalker::visit(PTree::NamespaceSpec *node)
   PTree::Node *pIdentifier = PTree::second(node);
   PTree::Node *pBody = PTree::third(node);
 
-  if (my_links) my_links->span(pNamespace, "file-keyword");
+  if (my_links) my_links->span(pNamespace, "keyword");
   else update_line_number(node);
 
   // Start the namespace
@@ -473,7 +466,7 @@ void SWalker::visit(PTree::NamespaceSpec *node)
   else ns = my_builder->start_namespace(my_file->filename(), NamespaceAnon);
 
   // Add comments
-  add_comments(ns, dynamic_cast<PTree::NamespaceSpec*>(node));
+  add_comments(ns, static_cast<PTree::NamespaceSpec*>(node));
   if (my_links && PTree::first(pIdentifier)) my_links->link(pIdentifier, ns);
 
   // Translate the body
@@ -497,7 +490,7 @@ std::vector<Inheritance*> SWalker::translate_inheritance_spec(PTree::Node *node)
     for (int i = 0; i != PTree::length(node->car()) - 1; ++i)
     {
       attributes[i] = parse_name(PTree::nth(node->car(), i));
-      if (my_links) my_links->span(PTree::nth(node->car(), i), "file-keyword");
+      if (my_links) my_links->span(PTree::nth(node->car(), i), "keyword");
     }
     // look up the parent type
     PTree::Node *name = PTree::last(node->car())->car();
@@ -562,17 +555,17 @@ void SWalker::visit(PTree::ClassSpec *node)
     // [ class|struct <name> <inheritance> [{ body }] ]
     pName = PTree::nth(node, 1);
     pInheritance = PTree::nth(node, 2);
-    pBody = dynamic_cast<PTree::ClassBody *>(PTree::nth(node, 3));
+    pBody = static_cast<PTree::ClassBody *>(PTree::nth(node, 3));
   }
   else if (size == SizeAnonClass)
     // An anonymous struct. OpenC++ encodes us a unique
     // (may be qualified if nested) name
     // [ struct [nil nil] [{ ... }] ]
-    pBody = dynamic_cast<PTree::ClassBody *>(PTree::nth(node, 2));
+    pBody = static_cast<PTree::ClassBody *>(PTree::nth(node, 2));
   else
     throw nodeERROR(node, "Class node has bad length: " << size);
 
-  if (my_links) my_links->span(pClass, "file-keyword");
+  if (my_links) my_links->span(pClass, "keyword");
   else update_line_number(node);
 
   // Create AST.Class object
@@ -590,8 +583,8 @@ void SWalker::visit(PTree::ClassSpec *node)
     LOG("Specialization?");
     nodeLOG(node);
     LOG("encname:"<< enc);
-    Types::Parameterized* param = dynamic_cast<Types::Parameterized*>(my_decoder->decodeTemplate());
-    // If a non-type param was found, it's name will be '*'
+    Types::Parameterized* param = my_decoder->decodeTemplate();
+    // If a non-type param was found, its name will be '*'
     for (size_t i = 0; i < param->parameters().size(); i++)
       if (Types::Dependent* dep = dynamic_cast<Types::Dependent*>(param->parameters()[i]))
       {
@@ -902,7 +895,6 @@ void SWalker::visit(PTree::Declaration *node)
   STrace trace("SWalker::visit(PTree::Declaration*)");
   // Link any comments added because we are inside a function body
   if (my_links) find_comments(node);
-
   update_line_number(node);
 
   my_declaration = node;
@@ -1087,9 +1079,17 @@ SWalker::translate_function_declarator(PTree::Node *decl, bool is_const)
     // Name is same as realname, but with parameters added
     std::string name = realname + format_parameters(params);
     // Append const after params if this is a const function
-    if (is_const) name += "const";
+  
+    // Figure out postmodifiers
+    std::vector<std::string> postmod;
+    if (is_const) 
+    {
+      name += "const";
+      postmod.push_back("const");
+    }
+
     // Create AST::Function object
-    func = my_builder->add_function(my_lineno, name, premod, returnType, realname, is_template);
+    func = my_builder->add_function(my_lineno, name, premod, returnType, postmod, realname, is_template);
     func->parameters() = params;
   }
   add_comments(func, my_declaration);
@@ -1170,7 +1170,7 @@ SWalker::translate_variable_declarator(PTree::Node *decl, bool /* is_const */)
     {
       // Link the const keyword
       if (*p->car() == "const")
-        my_links->span(p->car(), "file-keyword");
+        my_links->span(p->car(), "keyword");
       p = PTree::rest(p);
     }
     if (p)
@@ -1356,7 +1356,7 @@ SWalker::translate_type_specifier(PTree::Node *tspec)
 void SWalker::visit(PTree::Typedef *node)
 {
   STrace trace("SWalker::visit(Typedef*)");
-  if (my_links) my_links->span(PTree::first(node), "file-keyword");
+  if (my_links) my_links->span(PTree::first(node), "keyword");
   /* PTree::Node *tspec = */
   translate_type_specifier(PTree::second(node));
   my_declaration = node;
@@ -1489,7 +1489,7 @@ void SWalker::visit(PTree::AccessSpec *node)
     add_comments(builtin, comments);
   }
   my_builder->set_access(axs);
-  if (my_links) my_links->span(PTree::first(node), "file-keyword");
+  if (my_links) my_links->span(PTree::first(node), "keyword");
 }
 
 /* Enum Spec
@@ -1499,7 +1499,7 @@ void SWalker::visit(PTree::EnumSpec *node)
 {
   STrace trace("SWalker::visit(PTree::EnumSpec*)");
   //update_line_number(spec);
-  if (my_links) my_links->span(PTree::first(node), "file-keyword");
+  if (my_links) my_links->span(PTree::first(node), "keyword");
   if (!PTree::second(node))
   {
     return; /* anonymous enum */
@@ -1559,9 +1559,9 @@ void SWalker::visit(PTree::EnumSpec *node)
 void SWalker::visit(PTree::UsingDirective *node)
 {
   STrace trace("SWalker::visit(PTree::UsingDirective*)");
-  if (my_links) my_links->span(PTree::first(node), "file-keyword");
+  if (my_links) my_links->span(PTree::first(node), "keyword");
   PTree::Node *p = PTree::rest(node);
-  if (my_links) my_links->span(PTree::first(p), "file-keyword");
+  if (my_links) my_links->span(PTree::first(p), "keyword");
   // Find namespace to alias
   p = PTree::rest(p);
 
@@ -1612,7 +1612,7 @@ void SWalker::visit(PTree::UsingDirective *node)
 void SWalker::visit(PTree::UsingDeclaration *node)
 {
   STrace trace("SWalker::visit(PTree::UsingDeclaration*)");
-  if (my_links) my_links->span(PTree::first(node), "file-keyword");
+  if (my_links) my_links->span(PTree::first(node), "keyword");
   PTree::Node *p = PTree::rest(node);
   // Find name that we are looking up, and make a new ptree list for linking it
   PTree::Node *p_name = PTree::snoc(0, p->car());
