@@ -1,4 +1,4 @@
-"""Package for linking modules.
+"""AST processors.
 
 There is much that can be done to an AST to manipulate it. This includes:
 merging multiple AST trees, mapping links to different languages, pruning the
@@ -8,9 +8,10 @@ groupings."""
 from Linker import *
 from ScopeStripper import *
 from NameMapper import *
-from Comments import *
+from MacroFilter import *
+from ModuleFilter import *
 from XRefCompiler import *
-from EmptyModuleRemover import *
 from AccessRestrictor import *
 from TypeMapper import *
 from LanguageMapper import *
+import Comments
