@@ -21,6 +21,8 @@ Lexer::Lexer(Buffer *buffer, int tokenset)
   my_keywords["break"] = Token::BREAK;
   my_keywords["case"] = Token::CASE;
   my_keywords["char"] = Token::CHAR;
+  // FIXME: Add support for _Complex to Parser.
+  my_keywords["_Complex"] = Token::Ignore;
   my_keywords["const"] = Token::CONST;
   my_keywords["continue"] = Token::CONTINUE;
   my_keywords["default"] = Token::DEFAULT;
@@ -81,6 +83,7 @@ Lexer::Lexer(Buffer *buffer, int tokenset)
     my_keywords["__alignof__"] = Token::SIZEOF;
     my_keywords["__asm__"] = Token::ATTRIBUTE;
     my_keywords["__attribute__"] = Token::ATTRIBUTE;
+    my_keywords["__builtin_offsetof"] = Token::OFFSETOF;
     my_keywords["__builtin_va_arg"] = Token::EXTENSION; // Is this correct ?
     my_keywords["__complex__"] = Token::Ignore;
     my_keywords["__const"] = Token::CONST;
@@ -473,14 +476,33 @@ bool Lexer::read_str_const(unsigned long top)
     }
     else if(c == '"')
     {
+      // We are past one string literal token now.
+      // Any following whitespace needs to be skipped
+      // before looking for anything else.
       unsigned long pos = my_buffer->position() + 1;
-      int nline = 0;
-      do
+      while (true)
       {
-	c = my_buffer->get();
-	if(c == '\n') ++nline;
-      } while(is_blank(c) || c == '\n');
-
+	int nline = 0;
+	// Consume whitespace.
+	do
+	{
+	  c = my_buffer->get();
+	  if(c == '\n') ++nline;
+	} while(is_blank(c) || c == '\n');
+	// Consume comment.
+	if (c == '/')
+	{
+	  char d = my_buffer->get();
+	  if (d == '/' || d == '*')
+	    read_comment(d, my_buffer->position() - 2);
+	  else
+	  {
+	    my_buffer->unget();
+	    break;
+	  }
+	}
+	else break;
+      }
       if(c == '"')
 	/* line_number += nline; */ ;
       else
@@ -587,7 +609,7 @@ Token::Type Lexer::screen(const char *identifier, size_t length)
 Token::Type Lexer::read_separator(char c, unsigned long top)
 {
   char c1 = my_buffer->get();
-
+  if (c1 == '\0') return Token::BadToken;
   my_token.length = 2;
   if(c1 == '=')
   {
